package abcex

import (
	"fmt"
	"math"
)

var decToAbcex36 = map[int]string{
	0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7",
	8: "8", 9: "9", 10: "a", 11: "b", 12: "c", 13: "d", 14: "e",
	15: "f", 16: "g", 17: "h", 18: "i", 19: "j", 20: "k", 21: "l",
	22: "m", 23: "n", 24: "o", 25: "p", 26: "q", 27: "r", 28: "s",
	29: "t", 30: "u", 31: "v", 32: "w", 33: "x", 34: "y", 35: "z",
}

var decToAbcex62 = map[int]string{
	0: "0", 1: "1", 2: "2", 3: "3", 4: "4", 5: "5", 6: "6", 7: "7",
	8: "8", 9: "9", 10: "A", 11: "B", 12: "C", 13: "D", 14: "E",
	15: "F", 16: "G", 17: "H", 18: "I", 19: "J", 20: "K", 21: "L",
	22: "M", 23: "N", 24: "O", 25: "P", 26: "Q", 27: "R", 28: "S",
	29: "T", 30: "U", 31: "V", 32: "W", 33: "X", 34: "Y", 35: "Z",
	36: "a", 37: "b", 38: "c", 39: "d", 40: "e", 41: "f", 42: "g",
	43: "h", 44: "i", 45: "j", 46: "k", 47: "l", 48: "m", 49: "n",
	50: "o", 51: "p", 52: "q", 53: "r", 54: "s", 55: "t", 56: "u",
	57: "v", 58: "w", 59: "x", 60: "y", 61: "z",
}

var (
	BASE36 int = 36
	BASE62 int = 62
)

func getDict(base int) map[int]string {
	if base == 36 {
		return decToAbcex36
	} else if base == 62 {
		return decToAbcex62
	}

	panic("Invalid base")
}

func reverse(s string) string {
	runes := []rune(s)
	for i, j := 0, len(runes)-1; i < j; i, j = i+1, j-1 {
		runes[i], runes[j] = runes[j], runes[i]
	}
	return string(runes)
}

func flipMap(dict map[int]string) map[string]int {
	flipped := make(map[string]int)
	for i, val := range dict {
		flipped[val] = i
	}

	return flipped
}

func Encode(number int64, base int) string {
	if number == 0 {
		return "0"
	}

	result := ""
	negative := false
	if number < 0 {
		number = number * (-1)
		negative = true
	}

	for number > 0 {
		result = fmt.Sprintf("%s%s", getDict(base)[int(number%int64(base))], result)
		number /= int64(base)
	}

	if negative {
		return "-" + result
	}

	return result
}

func Decode(str string, base int) int64 {
	var result int64
	result = 0
	abcexToDec := flipMap(getDict(base))
	negative := false
	if string(str[0]) == "-" {
		negative = true
		str = str[1:]
	}

	str = reverse(str)

	for i, c := range str {
		result = int64(math.Pow(float64(base), float64(i)))*int64(abcexToDec[string(c)]) + result
	}

	if negative {
		return result * (-1)
	}

	return result
}

func Convert(str string, fromBase int, toBase int) string {
	return Encode(Decode(str, fromBase), toBase)
}
